/**
 * Plugin: Apply values to gaps in data
 * Looks for `applyGapValue` setting in chart config
 */
AmCharts.addInitHandler(function(chart) {

    /**
     * Do nothing if chart is not-date based or if `applyGapValue` not set
     */
    if (typeof chart.applyGapValue === "undefined" || typeof chart.categoryAxis === "undefined" || chart.categoryAxis.parseDates !== true)
        return;

    /**
     * Build a new data set
     */
    AmCharts.applyGapValue.processData(chart, false);


}, ["serial"]);

/**
 * Create a reusable object
 * @type {Object}
 */
AmCharts.applyGapValue = {

    /**
     * Resolves any variable into a Date object
     */
    "getDate": function(val, format) {
        if (typeof val === "string" && format !== undefined) {
            // try parse date from string using dataDateFormat
            return AmCharts.stringToDate(val, format);
        } else {
            // last resort: dump everything into Date constructor
            // and let browser handle it
            return new Date(val);
        }
    },

    /**
     * Formats the date into the format used in data
     */
    "formatDate": function(date, sampleFormat, format) {
        if (typeof sampleFormat === "string" && format !== undefined) {
            return AmCharts.formatDate(date, format);
        } else if (typeof sampleFormat === "number") {
            return date.getTime();
        }
        return date;
    },

    /**
     * Returns chart's minPeriod in milliseconds
     */
    "getPeriod": function(minPeriod) {
        minPeriod = minPeriod || "DD";
        var period = AmCharts.extractPeriod(minPeriod);
        return AmCharts.getPeriodDuration(period.period, period.count)
    },

    /**
     * Adds periods to a Date object
     */
    "addPeriod": function(date, minPeriod, count) {
        minPeriod = minPeriod || "DD";
        var period = AmCharts.extractPeriod(minPeriod);
        var newDate = new Date(date);
        switch (period.period) {
            case "YYYY":
                newDate.setFullYear(newDate.getFullYear() + count);
                break;
            case "MM":
                newDate.setFullYear(newDate.getFullYear(), newDate.getMonth() + count);
                break;
            case "DD":
                newDate.setFullYear(newDate.getFullYear(), newDate.getMonth(), newDate.getDate() + count);
                break;
            case "hh":
                newDate.setHours(newDate.getHours() + count);
                break;
            case "mm":
                newDate.setHours(newDate.getHours(), newDate.getMinutes() + count);
                break;
            case "ss":
                newDate.setHours(newDate.getHours(), newDate.getMinutes(), newDate.getSeconds() + count);
                break;
            case "fff":
                newDate.setTime(newDate.getTime() + count);
                break;
        }
        return newDate;
    },

    /**
     * Processes chart data - fills the gaps with value
     */
    "processData": function(chart, validateData) {

        // get references
        var currentData = chart.dataProvider;
        var categoryField = chart.categoryField;
        var dataDateFormat = chart.dataDateFormat;

        // check if data set is maybe empty
        if (!AmCharts.ifArray(currentData) || currentData.length === 0)
            return;

        // get first and last dates, min period in milliseconds
        var firstDate = AmCharts.applyGapValue.getDate(
                currentData[0][categoryField],
                dataDateFormat),
            lastDate = AmCharts.applyGapValue.getDate(
                currentData[currentData.length - 1][categoryField],
                dataDateFormat),
            sampleFormat = currentData[0][categoryField];

        /**
         * Build a new set of data
         */
        var data = [];
        for (var i = 0; i < currentData.length; i++) {

            // add current data point
            data.push(currentData[i]);

            // check if maybe we have a gap
            if ((i + 1) < currentData.length) {
                var nextPoint = currentData[i + 1];
                var nextDate = AmCharts.applyGapValue.getDate(nextPoint[categoryField]);
                var nextSupposedDate = AmCharts.applyGapValue.addPeriod(
                    AmCharts.applyGapValue.getDate(currentData[i][categoryField]),
                    chart.categoryAxis.minPeriod,
                    1);
                while (nextSupposedDate < nextDate) {
                    //console.log(currentData[ i ][ categoryField ], nextSupposedDate, nextDate);
                    var dp = {};
                    dp[categoryField] = AmCharts.applyGapValue.formatDate(nextSupposedDate, sampleFormat, chart.dataDateFormat);
                    for (var g = 0; g < chart.graphs.length; g++) {
                        dp[chart.graphs[g].valueField] = chart.applyGapValue;
                    }
                    data.push(dp);
                    //console.log( dp );
                    nextSupposedDate = AmCharts.applyGapValue.addPeriod(nextSupposedDate, chart.categoryAxis.minPeriod, 1);
                }
            }

        }

        // update dataProvider
        chart.dataProvider = data;

        // should we revalidate the data?
        if (validateData)
            chart.validateData();
    }
};
